#include <p18cxxx.h>
#include <i2c.h>
#include "ds1307.h"
void DS_settime_array(char* time){
	DS_settime(&time[0], &time[1], &time[2]);
}
void DS_settime(char* hour, char* min, char*sec)
{
	// es wird keiner AM/PM Darstellung verwendet
	char raw[]= {0,0,0};
	char tmp = 0; 
	DS_Read(raw, 1,0x00); // Lese 1. Byte, wegen CH
	
	if((*hour>=0) && (*hour<24)){
		if((*hour>10)&&(*hour<20) ){
			raw[2] &= 0x10;
			*hour -= 10;
		}
		if(*hour>=20){
			raw[2] &= 0x20;	
			*hour -=20;
		}
		raw[2] |= 0x40; //24 Stunden Darstellung
		raw[2] |= *hour;
		
	}
	if((*min>=0) && (*min<60)){
		reconvert(min, &raw[1]);
	}
	if((*sec>=0) && (*sec<60)){
		tmp = raw[0] & 0x80; //CH
		raw[0]&= 0x7F;
		reconvert(sec,&raw[0]);
		raw[0]|=tmp;
		
	}
	DS_Write(raw,3,0x00);
}
void DS_setdate(char *weekday, char *day, char *month, char *year)
{
	char raw[]= {0,0,0,0};
	char tmp = 0; 
	if((*weekday>0) && (*weekday<=7)){
		raw[0] |= *weekday;
	}
	if((*day>0) && (*day<31)){
		reconvert(day, &raw[1]);
	}
	if((*month>0) && (*month<=12)){
		reconvert(month, &raw[2]);
	}
	if((*year>0) && (*year<=99)){
		reconvert(year, &raw[3]);
	}
	DS_Write(raw,4,0x03);

}
void DS_gettime_array(char *time){
	DS_gettime(&time[0], &time[1], &time[2]);
}
void DS_gettime(char *hour, char *min, char *sec)
{
	char raw[] = {0,0,0}, tmp;
	DS_Read(raw,4,0x00);
	//Sekunden
	raw[0] &= 0x7F; //CH Bit lschen, falls gesetzt
	convert(&raw[0],sec);
	//Minuten
	convert(&raw[1],min);
	//Stunden
	raw[2] &= 0x3F; // Bit 6+7 lschen, falls gesetzt
	convert(&raw[2],hour);
	
}
void DS_gettime_split(char *chour, char *hour, char *cmin, char *min, char *csec, char *sec){
	char raw[] = {0,0,0};
	DS_Read(raw,4,0x00);
	*csec = (raw[0]& 0x70)>>4;
	*sec = (raw[0]& 0x0F);
	*cmin = (raw[1]& 0x70)>>4;
	*min = (raw[1]& 0x0F);
	*chour = (raw[2]& 0x30)>>4;
	*hour = (raw[2]& 0x0F);
}

void DS_getdate(char *weekday, char *day, char *month, char *year)
{
	char raw[] = {0,0,0,0}, tmp;
	DS_Read(raw,3,0x03);
	// Wochentag
	*weekday = raw[0];
	// Tag
	convert(&raw[1],day);
	// Monat
	convert(&raw[2],month);
}
void DS_Run(void){
	char tmp []={0},t;
	DS_Read (tmp,1,0x00);
	t=tmp[0];
	tmp[0] &= 0x7F; // CH lschen
	DS_Write(tmp,1 ,0x00);
}
void DS_Stop(void){
	char tmp [];
	DS_Read (tmp,1,0x00);
	tmp[0] |= 0x80; // CH setzten
	DS_Write(tmp, 1, 0x00);
}
void DS_Out(char mode) //Mode: OneHz/FourkHz/EightkHz/ThirteenkHz/OutHigh/OutLow
{
	char * pmode = &mode;
	DS_Write(pmode, 1,0x07); //Schreibt den Modus in das Control Register 0x08
}
void DS_Read(char* data, char length, char wordaddr)
{
	int i;
	unsigned char t;
	OpenI2C(MASTER, SLEW_ON);
	SSPADD = 9;
	IdleI2C();
	StartI2C();
	while(SSPCON2bits.SEN==1);
	IdleI2C();
	WriteI2C(0xD0);
	IdleI2C();
	WriteI2C(wordaddr);
	IdleI2C();
	StartI2C();
	IdleI2C();
	WriteI2C(0xD1);
	IdleI2C();
	// An nachfolgender Stelle geht es nicht weiter.
	
	t = ReadI2C(); // t zu Testzwecken
	*data = t;
	for(i=1; i<length; i++)
	{
		AckI2C();
		data++;
		t = ReadI2C();
		*data = t;	
	}
	NotAckI2C();
	StopI2C();
	IdleI2C();
	CloseI2C();
	
}
void DS_Write(char* data, char lenght, char wordaddr)
{
	int i;
	char t;
	OpenI2C(MASTER, SLEW_ON);
	SSPADD = 9;
	IdleI2C();
	StartI2C();
	while(SSPCON2bits.SEN==1);
	WriteI2C(0xD0);
	IdleI2C();	
	WriteI2C(wordaddr);
	IdleI2C();	
	for(i=0; i<lenght; i++){
		t = *data;
		WriteI2C(*data);
		data++;
		IdleI2C();	
	}	
	StopI2C();
	IdleI2C();	
	CloseI2C();
}
void convert(char *rawdata, char *data){
	char tmp = (*rawdata >>4)*10;
	*data = tmp + (*rawdata & 0x0F);
}
void reconvert(char *data, char *raw)
{
	char tmp = *data%10; // Teilungrest durch 10 ermitteln (34%10 = 4)
	*data -= tmp;	 // (34 - 4 = 30)
	*raw|= *data<<4; // 10 Minuten
	*raw|= tmp; // 1  Minuten
}